/*
** BaseMod.c:
**
** Small C Demo program to show writing of linux modules
**
** written by Peter Weiss 9/96
**
** based on base.c as provided by modules-2.0.0 package
**
*/

#define MODULE_NAME "BaseMod"

#include <linux/config.h>
#include <linux/module.h>

#ifdef MODVERSIONS
#include <linux/modversions.h>

#ifndef __GENKSYMS__            /* if this is defined we are generating the */
#include "BaseMod.ver"          /* symbol version file                      */
#endif 

#endif

int  init_module(void);
void cleanup_module(void);

/*
** This is declared static and will be explicitly exported to the
** kernels symbol table using register_symtab().
*/
static void BaseModMessage(const char *message)
{
    printk("BaseMod's message is '%s'\n", message);
}


/*
** This is the number that the TestDevice module uses to determine it's
** major number. Well it could also be provided as define in the source there,
** but this way we can the how stacking of modules works.
** The number 42 is taken as mentioned in
** /usr/src/linux/Documentation/devices.txt, where it's reserved for testing
** purposes.
*/

static int BaseModMajorNumber = 42;
/*
** We like to provide the following symbols to the public:
*/

static struct symbol_table BaseMod_syms = {
#include <linux/symtab_begin.h>
    X(BaseModMajorNumber),
#include <linux/symtab_end.h>
};


/*
** This is the "constructor" of the module, called by the kernel after it is
** loaded into kernel space.
*/
int init_module()
{
    register_symtab(&BaseMod_syms);
    BaseModMessage("Symbols are registered");

    return 0;
}


/*
** This is the "destructor" of the module, called by the kernel before it is
** removed.
** In this example there is not much to do...
*/
void cleanup_module()
{
    return;
}

/*
** Local Variables:
** compile-command: "cc -DMODULE -D__KERNEL__ -c -O BaseMod.c -o BaseMod.o"
** c-macro-preprocessor: "/lib/cpp -C -DMODULE -D__KERNEL__ "
** End:
*/

/****************************** End of BaseMod.C ******************************/
