/*
** TestDevice.c:
**
** Small C Demo program to show writing of linux modules. Well, I know it's
** useless but it shows how the stuff works.
**
** written by Peter Weiss 9/96
**
** based drv_hello.c provided by modules-2.0.0 package
** compiled and tested on a linux-2.0.22 system, but should work at least with
** versions 1.3.xx and above
**
*/

#define MODULE_NAME "TestDevice"

#ifndef MAXBUFSIZE
#define MAXBUFSIZE 512
#endif

/* Kernel include files */

#include <linux/config.h>
#include <linux/module.h>

#include <linux/errno.h>
#include <linux/fs.h>
#include <linux/major.h>
#include <asm/segment.h>
#include <linux/kernel.h>
#include <linux/signal.h>

#include <linux/config.h>
#include <linux/module.h>

#ifdef MODVERSIONS
#include <linux/modversions.h>  /* We do not need an own .ver file cause we  */
#include "BaseMod.ver"          /* don't have any global symbols defined,    */
#endif                          /* but we need those of BaseMod             */

/* The buffer that is accessable from outside the kernel through this module */
static unsigned char DevBuffer[MAXBUFSIZE];

/* This symbol will be provided by the BaseMod module: */
extern int BaseModMajorNumber;
static int majorNumber;

/*
** Read requests on the Testdevice: The DevBuffer is copied byte by byte
** to user space until an end of string is found.
*/
static int TestDev_read(struct inode *node, struct file *file,
                        char *buf, int count)
{
    int left, devpos = ((int) file->f_pos) % (MAXBUFSIZE - 1);

    for (left = count; left > 0; left--) {
        put_fs_byte(DevBuffer[devpos], buf++);

        if (DevBuffer[devpos++] == '\0')
            break;

        if (devpos == MAXBUFSIZE)
            devpos = 0;
    }

    file->f_pos += count - left;

    return count - left;
}



/*
** Write buf to DevBuffer assuming it's something like a string, append a
** null byte at the end if possible.
*/
static TestDev_write(struct inode *inode, struct file *file, const char *buf,
                     int count)
{
    int left, devpos = ((int) file->f_pos) % (MAXBUFSIZE - 1);

    for (left = count; left > 0; left--) {
        *(DevBuffer + devpos) = get_fs_byte(buf++);

        if (++devpos == MAXBUFSIZE)
            devpos = 0;
    }

    if (devpos < MAXBUFSIZE)
        DevBuffer[devpos] = '\0';

    file->f_pos += count;

    return count;
}


/*
 * support seeks on the device
 */
static int TestDev_lseek(struct inode *inode, struct file *file,
                         off_t offset, int orig)
{
    switch (orig) {
        case 0:
            file->f_pos = offset;
            return file->f_pos;
        case 1:
            file->f_pos += offset;
            return file->f_pos;

        default: /* how does the device have an end? */
            return -EINVAL;
    }
}


/*
 * Our special open code.
 * MOD_INC_USE_COUNT make sure that the driver memory is not freed
 * while the device is in use.
 */
static int TestDev_open(struct inode* inode, struct file* filep)
{
    MOD_INC_USE_COUNT;

    return 0;
}

/*
 * Now decrement the use count.
 */
static void TestDev_close(struct inode* inode, struct file* filep)
{
    MOD_DEC_USE_COUNT;
}


static struct file_operations TestDevFops = {
    TestDev_lseek,
    TestDev_read,
    TestDev_write,
    NULL,        /* no readdir            function */
    NULL,        /* no select             function */
    NULL,        /* no ioctl              function */
    NULL,        /* no mmap               function */
    TestDev_open,
    TestDev_close,
    NULL,        /* no fsync              function */
    NULL,        /* no fasync             function */
    NULL,        /* no check_media_change function */
    NULL,        /* no revalidate         function */
};

/*
** And now the modules code and kernel interface.
*/

extern int printk(const char* fmt, ...);

int init_module()
{
    int i;
                               /* Access the major number provided by BaseMod */
    majorNumber = BaseModMajorNumber;

    printk("%s: init_module called with device major number %d\n", MODULE_NAME,
           majorNumber);

    if (register_chrdev(majorNumber, "testdev", &TestDevFops)) {
        printk("register_chrdev failed!\n");
        return -EIO;
    } else
        printk("%s: Module successfull activated\n", MODULE_NAME);

    return 0;               /* Tell the kernel the module is installed */
}


void cleanup_module()
{
    printk("%s: cleanup_module called\n", MODULE_NAME);

    if (unregister_chrdev(majorNumber, "testdev") != 0)
        printk("%s: cleanup_module failed\n",    MODULE_NAME);
    else
        printk("%s: cleanup_module succeeded\n", MODULE_NAME);
}


/*
** Local Variables:
** compile-command:"cc -DMODULE -D__KERNEL__ -c -O TestDevice.c -o TestDevice.o"
** c-macro-preprocessor: "/lib/cpp -C -DMODULE -D__KERNEL__ "
** End:
*/

/***************************** End of TestDevice.C*****************************/
